#include "cardappz.h"

int DesfireSelectCorpid(CARDAPPZ_CTX_ST * ctx)
{
  int rc;
  unsigned char key[16];

  /* Select the application directory */
  rc = DesfireSelectApplication(ctx, CORPID_DESFIRE_AID);
  if (rc)
    return rc;

  /* Compute the diversified key 1 */
  rc = ComputeDivKey(ctx, key, CORPID_MASTER_KEY1);
  if (rc)
    return rc;

  /* Get authenticated with key 1 */
  rc = DesfireAuthenticate(ctx, 1, key);
  if (rc)
    return rc;

  return CARDAPPZ_OK;
}

int DesfireReadCorpidIdentity(CARDAPPZ_CTX_ST * ctx,
                              CORPID_IDENTITY_ST * identity)
{
  int length = sizeof(CORPID_IDENTITY_ST);

  return DesfireReadData(ctx, CORPID_DESFIRE_FID_IDENTITY,
                         DESFIRE_COMM_CIPHER, 0, 0,
                         (unsigned char *) identity, &length);
}

int DesfireReadCorpidFingerprints(CARDAPPZ_CTX_ST * ctx,
                                  unsigned char
                                  fingerprints[CORPID_FINGERPRINTS_MAX_SZ])
{
  int length = CORPID_FINGERPRINTS_MAX_SZ;

  return DesfireReadData(ctx, CORPID_DESFIRE_FID_FINGERPRINTS,
                         DESFIRE_COMM_CIPHER, 0, 0, fingerprints, &length);
}

int DesfireReadCorpidPicture(CARDAPPZ_CTX_ST * ctx,
                             unsigned char picture[CORPID_PICTURE_MAX_SZ])
{
  int length = CORPID_PICTURE_MAX_SZ;

  return DesfireReadData(ctx, CORPID_DESFIRE_FID_PICTURE,
                         DESFIRE_COMM_CIPHER, 0, 0, picture, &length);
}

int DesfireReadCorpidAccessControl(CARDAPPZ_CTX_ST * ctx,
                                   unsigned char accesscontrol[48])
{
  int length = 48;

  return DesfireReadData(ctx, CORPID_DESFIRE_FID_ACCESSCONTROL,
                         DESFIRE_COMM_PLAIN, 0, 0, accesscontrol, &length);
}

int DesfireReadCorpidOptionsSign(CARDAPPZ_CTX_ST * ctx,
                                 CORPID_OPTIONSSIGN_ST * optionssing)
{
  int length = sizeof(CORPID_OPTIONSSIGN_ST);

  return DesfireReadData(ctx, CORPID_DESFIRE_FID_OPTIONSSIGN,
                         DESFIRE_COMM_PLAIN, 0, 0,
                         (unsigned char *) optionssing, &length);
}

int DesfireWriteCorpid(CARDAPPZ_CTX_ST * ctx, CORPID_IDENTITY_ST * identity,
                       unsigned char
                       fingerprints[CORPID_FINGERPRINTS_MAX_SZ],
                       unsigned char picture[CORPID_PICTURE_MAX_SZ],
                       unsigned char accesscontrol[48],
                       CORPID_OPTIONSSIGN_ST * optionssing)
{
  int rc;
  unsigned char key[16];

  /* Select the application directory */
  rc = DesfireSelectApplication(ctx, CORPID_DESFIRE_AID);
  if (rc)
    return rc;

  /* Compute the diversified key 0 */
  rc = ComputeDivKey(ctx, key, CORPID_MASTER_KEY0);
  if (rc)
    return rc;

  /* Get authenticated with key 0 */
  rc = DesfireAuthenticate(ctx, 0, key);
  if (rc)
    return rc;

  /* Write identity file */
  if (identity != NULL)
  {
    rc =
      DesfireWriteData(ctx, CORPID_DESFIRE_FID_IDENTITY, DESFIRE_COMM_CIPHER,
                       0, sizeof(CORPID_IDENTITY_ST),
                       (unsigned char *) identity);
    if (rc)
      return rc;
  }

  /* Write fingerprints file */
  if (fingerprints != NULL)
  {
    rc =
      DesfireWriteData(ctx, CORPID_DESFIRE_FID_FINGERPRINTS,
                       DESFIRE_COMM_CIPHER, 0, CORPID_FINGERPRINTS_MAX_SZ,
                       fingerprints);
    if (rc)
      return rc;
  }

  /* Write picture file */
  if ((picture != NULL) && (optionssing != NULL))
  {
    /* Size of picture must be retrieved in options */
    unsigned short size = CtoHs(optionssing->PictureSize);

    rc =
      DesfireWriteData(ctx, CORPID_DESFIRE_FID_PICTURE, DESFIRE_COMM_CIPHER,
                       0, size, picture);
    if (rc)
      return rc;
  }

  /* Write access control file */
  if (accesscontrol != NULL)
  {
    rc =
      DesfireWriteData(ctx, CORPID_DESFIRE_FID_ACCESSCONTROL,
                       DESFIRE_COMM_PLAIN, 0, 48, accesscontrol);
    if (rc)
      return rc;
  }

  /* Write options and signature file */
  if (optionssing != NULL)
  {
    rc =
      DesfireWriteData(ctx, CORPID_DESFIRE_FID_OPTIONSSIGN,
                       DESFIRE_COMM_PLAIN, 0, sizeof(CORPID_OPTIONSSIGN_ST),
                       (unsigned char *) optionssing);
    if (rc)
      return rc;
  }

  return CARDAPPZ_OK;
}

/* Formatting */

/* ---------- */
int DesfireFormatCorpid(CARDAPPZ_CTX_ST * ctx)
{
  int rc;
  unsigned char key[16];

  /* Select application 0 */
  rc = DesfireSelectApplication(ctx, 0);
  if (rc)
    return rc;

  /* Get authenticated with master key */
  rc = DesfireAuthenticate(ctx, 0, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Create the identity application with 2 keys */
  rc = DesfireCreateApplication(ctx, CORPID_DESFIRE_AID, 0xE3, 2);
  if (rc)
    return rc;

  /* Select the application */
  rc = DesfireSelectApplication(ctx, CORPID_DESFIRE_AID);
  if (rc)
    return rc;

  /* Formatting is performed with (blank) key 0 */
  rc = DesfireAuthenticate(ctx, 0, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Create user data file */
  rc = DesfireCreateStdDataFile(ctx, CORPID_DESFIRE_FID_IDENTITY, DESFIRE_COMM_CIPHER, 0x1F0F, 128);  /* Ciphered ; Key 0 = R/W ; Key 1 = R */
  if (rc)
    return rc;

  /* Create fingerprints file */
  rc = DesfireCreateStdDataFile(ctx, CORPID_DESFIRE_FID_FINGERPRINTS, DESFIRE_COMM_CIPHER, 0x1F0F, 170);  /* Ciphered ; Key 0 = R/W ; Key 1 = R */
  if (rc)
    return rc;

  /* Create picture file */
  rc = DesfireCreateStdDataFile(ctx, CORPID_DESFIRE_FID_PICTURE, DESFIRE_COMM_CIPHER, 0x1F0F, 2464);  /* Ciphered ; Key 0 = R/W ; Key 1 = R */
  if (rc)
    return rc;

  /* Create access control file */
  rc = DesfireCreateStdDataFile(ctx, CORPID_DESFIRE_FID_ACCESSCONTROL, DESFIRE_COMM_PLAIN, 0xEF0F, 48); /* Plain ; Key 0 = R/W */
  if (rc)
    return rc;

  /*  Create options and signature file */
  rc = DesfireCreateStdDataFile(ctx, CORPID_DESFIRE_FID_OPTIONSSIGN, DESFIRE_COMM_PLAIN, 0xEF0F, 32); /* Plain ; Key 0 = R/W */
  if (rc)
    return rc;

  /* Compute the diversified key 0 */
  rc = ComputeDivKey(ctx, key, CORPID_MASTER_KEY0);
  if (rc)
    return rc;

  /* Put the new key 0 */
  rc = DesfireChangeKey(ctx, 0, key);
  if (rc)
    return rc;

  /* Get authenticated with (blank) key 1 */
  rc = DesfireAuthenticate(ctx, 1, DESFIRE_BLANK_KEY);
  if (rc)
    return rc;

  /* Compute the diversified key 1 */
  rc = ComputeDivKey(ctx, key, CORPID_MASTER_KEY1);
  if (rc)
    return rc;

  /* Put the new key 1 */
  rc = DesfireChangeKey(ctx, 1, key);
  if (rc)
    return rc;

  /* Done ! */
  return CARDAPPZ_OK;
}
